function Coh = SCoh_W(y,x,alpha,nfft,Noverlap,Window,opt,P)     
% Coh = SCoh_W(y,x,alpha,nfft,Noverlap,Window,opt,P)
% Welch's estimate of the Cyclic Spectral Coherence of signals y and x 
% at cyclic frequency alpha (0 <= 1/cycle <1):  
% opt = 'sym' : E{Y(f+a/2)X*(f-a/2)}/sqrt(E|Y(f+a/2)|E|X*(f-a/2)|)
% opt = 'asym': E{Y(f)X*(f-a)}/sqrt(E|Y(f)|E|X*(f-a)|)
% x and y are divided into K overlapping blocks (Noverlap taps), each of which is
% detrended, windowed and zero-padded to length nfft. Denoting by Nwind the window length, it is recommended to use 
% nfft = 2*NWind and Noverlap = 2/3*Nwind (hanning window) or Noverlap = 1/2*Nwind (half-sine window).
% Note : use analytic signal to avoid correlation between + and - frequencies
%
% -----
% Outputs
% -------
% Coh is a structure organized as follows:
%                   Coh.C = Cyclic Spectral Coherence
%                   Coh.S = Cyclic Spectral Density
%                   Coh.f = vector of frequencies
%                   Coh.K = number of blocks
%                   Coh.Varduc = Variance Reduction factor
%                   Coh = CPS_W(y,x,...,P) where P is a scalar between 0 and 1, 
%                   also returns Coh.thres the P*100% significance level for testing against H0: |Coh.S|^2 = 0.
% 
% --------------------------
% Ref.: J. Antoni, "Cyclic Spectral Analysis in Practice", Mechanical Systems and Signal Processing, Volume 21, Issue 2 , February 2007, Pages 597-630.
% --------------------------
% Author: J. Antoni, 05-2003
% Revised : 07-2004, 09-2007
% --------------------------

if nargin > 7 & (P>=1 | P<=0)
    error('P must be in ]0,1[ !!')
end

if length(Window) == 1
    Window = hanning(Window);
end
Window = Window(:);
n = length(x);
nwind = length(Window);
if nwind<=Noverlap,error('Window length must be > Noverlap');end
y = y(:);
x = x(:);

t = (0:n-1)';
if strcmp(opt,'sym') == 1
    y = y.*exp(-1i*pi*alpha*t);
    x = x.*exp(1i*pi*alpha*t);
else
    x = x.*exp(2i*pi*alpha*t);
end

Syx = CPS_W(y,x,0,nfft,Noverlap,Window,opt);    
Sy = CPS_W(y,y,0,nfft,Noverlap,Window,opt);     
Sx = CPS_W(x,x,0,nfft,Noverlap,Window,opt);     
K = Sx.K;   
f = Sx.f;   

Coh = Syx.S./sqrt(Sy.S.*Sx.S);

% variance reduction factor
Window = Window(:)/norm(Window);
Delta = nwind - Noverlap;
R2w = xcorr(Window);
k = nwind+Delta:Delta:min(2*nwind-1,nwind+Delta*(K-1));
if length(k) >1
    Var_Reduc = R2w(nwind)^2/K + 2/K*(1-(1:length(k))/K)*(R2w(k).^2);
else
    Var_Reduc = R2w(nwind)^2/K;
end

% threshold on square magnitude at P significance level under H0
if nargin > 7
    threshold = chi2inv(1-P,2)*Var_Reduc/2;
    %treshold = sqrt(3)*Var_Reduc;
    %treshold = treshold*sqrt(2)*erfinv(1-P);
end

% set up output parameters
if nargout == 0
    figure,newplot;
    subplot(211),plot(f(1:nfft/2+1),abs(Coh(1:nfft/2+1)).^2), grid on
    if nargin > 7,
        hold on,plot(f(1:nfft/2+1),threshold,':r'),
        title(['Spectral Coherence (square magnitude) and threshold at ',num2str(100*P),'% level of significance'])
    else
        title(['Spectral Coherence (square magnitude)'])
    end
    subplot(212),plot(f(1:nfft/2+1),angle(Coh(1:nfft/2+1))), grid on
    xlabel('[Hz]'),title('Phase')
else
    Coh.C = Coh;
    Coh.S = Syx.S;
    Coh.f = f;
    Coh.K = K;
    Coh.Varduc = Var_Reduc;
    if nargin > 7 
        Coh.thres = threshold;
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function x = chi2inv(p,v);
if nargin < 2, 
    error('Requires two input arguments.');
end
x = gaminv(p,v/2,2);
k = find(v < 0  |  round(v) ~= v);
if any(k)
    tmp  = NaN;
    x(k) = tmp(ones(size(k)));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function x = gaminv(p,a,b);

if nargin<3, 
    b=1;
end
x = zeros(size(p));
k = find(p<0 | p>1 | a <= 0 | b <= 0);
if any(k),
    tmp  = NaN;
    x(k) = tmp(ones(size(k)));
end
k0 = find(p == 0 & a > 0 & b > 0);
if any(k0),
    x(k0) = zeros(size(k0)); 
end
k1 = find(p == 1 & a > 0 & b > 0);
if any(k1), 
    tmp = Inf;
    x(k1) = tmp(ones(size(k1))); 
end
count_limit = 100;
count = 0;
k = find(p > 0  &  p < 1 & a > 0 & b > 0);
pk = p(k);
mn = a(k) .* b(k);
v = mn .* b(k);
temp = log(v + mn .^ 2); 
mu = 2 * log(mn) - 0.5 * temp;
sigma = -2 * log(mn) + temp;
xk = exp(norminv(pk,mu,sigma));
h = ones(size(pk)); 
while(any(abs(h) > sqrt(eps)*abs(xk))  &  max(abs(h)) > sqrt(eps)    ...
        & count < count_limit), 
    
    count = count + 1;
    h = (gamcdf(xk,a(k),b(k)) - pk) ./ gampdf(xk,a(k),b(k));
    xnew = xk - h;
    ksmall = find(xnew < 0);
    if any(ksmall),
        xnew(ksmall) = xk(ksmall) / 10;
        h = xk-xnew;
    end
    xk = xnew;
end
x(k) = xk;
if count == count_limit, 
    fprintf('\nWarning: GAMINV did not converge.\n');
    str = 'The last step was:  ';
    outstr = sprintf([str,'%13.8f'],h);
    fprintf(outstr);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function x = norminv(p,mu,sigma);

if nargin < 3, 
    sigma = 1;
end
if nargin < 2;
    mu = 0;
end
x = zeros(size(p));
k = find(sigma <= 0 | p < 0 | p > 1);
if any(k)
    tmp  = NaN;
    x(k) = tmp(ones(size(k))); 
end
k = find(p == 0);
if any(k)
    tmp  = Inf;
    x(k) = -tmp(ones(size(k)));
end
k = find(p == 1);
if any(k)
    tmp  = Inf;
    x(k) = tmp(ones(size(k))); 
end
k = find(p > 0  &  p < 1 & sigma > 0);
if any(k),
    x(k) = sqrt(2) * sigma(k) .* erfinv(2 * p(k) - 1) + mu(k);
end
