% Generate Frequency Domain EQ Coefficients to Equalize Noise Playback
% K. Nieman, 10/2/12

% File locations
probeSignalDir = '.';
probeSignalFile = 'probe_signal.csv';
lineSamplesDir = '.';
lineSamplesFile = 'rx_line_samples.bin';
outDir = '..';
outFile = 'eq_coefs.bin';

% Agilent 33120A probe sequence parameters
N = 16000;
seed = 1;
desiredFs = 400e3;

% Generate probe signal
fprintf('Generating probe signal...');
seed = 1;
randn('state',seed);
n = (0:N-1).';
x = chirp(n/400e3,0,N/400e3,200e3);
freq = 1/(N/desiredFs);
fprintf('DONE.\n');
fprintf('Probe signal saved to:\n    %s\n',fullfile(probeSignalDir,probeSignalFile));
fid = fopen(fullfile(probeSignalDir,probeSignalFile),'w+');
for k = 1:N
    fprintf(fid,'%d, %0.9f\n',n(k),x(k));
end
fclose(fid);

% Collect line samples while probe sequence is played
fprintf('Download probe signal to Agilent 33120A, set to BURST/ARB mode and set to continuous playback.\n  Collect line samples from MCR1 and save them to:\n    %s\n',fullfile(lineSamplesDir,lineSamplesFile));
fprintf('(Press any key when done.)\n');
pause;

fprintf('Loading line samples from:\n    %s\n',fullfile(lineSamplesDir,lineSamplesFile));
y = real(readSamplesFile(fullfile(lineSamplesDir,lineSamplesFile),'complex'));
y = y./25e3;

fprintf('Synchronizing samples...');
z = matchedFilter(y,x.*hamming(N));
[m,I] = max(abs(z));
z = circshift(z,N-(I - 1));
y = circshift(y,N-(I - 1));
fprintf('DONE.\n');

figure(1);
plot(n - N/2,20*log10(abs(circshift(z,N/2))),'b')
axis([-N/2 N/2-1 0 100]);

% Design equalizer based on ZF solution
fprintf('Building ZF equalizer...');
X = fft(x);
Y = fft(y);
H = Y./X;
fprintf('DONE.\n');

% Plot measured channel response
figure(2);
plot(1e-3*desiredFs*(1:N/2-1)/N,20*log10(abs(H(1:N/2-1))),'b')
title('measured channel response');
xlabel('{\itf} (kHz)');
ylabel('|{\itH}({\itf})| (dB)');
grid

% Polynomial fit magnitude response of receiver
L = 11;
P = polyfit(n,abs(H),L);
Hs = sum(repmat(n,1,L+1).^repmat((L:-1:0),N,1).*repmat(P,N,1),2);
plot(n,20*log10(abs(H)),'b',n,20*log10(Hs),'r');
fprintf('Saving equalizer coefficients to:\n    %s\n',...
    fullfile(outDir,outFile));

% Write EQ coefficients to file
fid = fopen(fullfile(outDir,outFile),'w+');
H_eq = 1./Hs;
H_eq(1:360) = 1;
H_eq(8001:end) = flipud(H_eq(1:8000));
H_eq = [real(H_eq.');imag(H_eq).'];
fwrite(fid,H_eq(:),'double');
fclose(fid);